@@ -144,9 +144,9 @@ class MatterDeviceController:
     @api_command(APICommand.GET_NODE)
     def get_node(self, node_id: int) -> MatterNodeData:
         """Return info of a single Node."""
-        node = self._nodes.get(node_id)
-        assert node is not None, "Node does not exist or is not yet interviewed"
-        return node
+        if node := self._nodes.get(node_id):
+            return node
+        raise NodeNotExists(f"Node {node_id} does not exist or is not yet interviewed")
 
     @api_command(APICommand.COMMISSION_WITH_CODE)
     async def commission_with_code(self, code: str) -> MatterNodeData:
@@ -517,7 +517,14 @@ class MatterDeviceController:
 
         if len(attr_subscriptions) > 50:
             # prevent memory overload on node and fallback to wildcard sub if too many
-            # individual subscriptions
+            # individual subscriptions (e.g. bridges)
+            attr_subscriptions = "*"  # type: ignore[assignment]
+
+        if not node.attribute_subscriptions:
+            # temp fix for backwards compatbility with HA releases below 2023.7
+            # fallback to wildcard subscriptions if we have no explicit
+            # node subscriptions defined.
+            # TODO: remove this after HA release 2023.8
             attr_subscriptions = "*"  # type: ignore[assignment]
 
         # check if we already have an subscription for this node,
